;;;; "Basic Choreography" -- just show off audio/video

(ns com.aurellem.run.basic-choreo
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run util music title save-corruption
                          bootstrap-0 bootstrap-1 image
                          ram-display final-cut))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])
  (:import java.io.File))

;; take over game
;; [X] silence noise
;; [X] preload audio/video programs
;; [X] run picture program to display pinkie pie mark
;; [X] play pony song 

(defn-memo silence-noise
  ([] (silence-noise (launch-main-bootstrap-program)))
  ([script]
     (->> script
          (write-RAM 0xFF1A [0 0 0])
          (relocate-main-bootstrap))))

(def music-program-start 0xC000)
(def image-program-start 0xCE00)

(defn-memo write-pony-music
  ([] (write-pony-music (silence-noise)))
  ([script]
     (let [bytes (midi-bytes
                  pony-csv
                  music-program-start
                  0xC400
                  0xC800
                  0xCC00)]
       (->> script
            (write-RAM (:address (:voice-1 bytes))
                       (:data    (:voice-1 bytes)))
            (write-RAM (:address (:voice-2 bytes))
                       (:data    (:voice-2 bytes)))
            (write-RAM (:address (:noise bytes))
                       (:data    (:noise bytes)))
            (write-RAM (:address (:kernel bytes))
                       (:data    (:kernel bytes)))))))

(defn-memo write-image-display-program
  ([] (write-image-display-program (write-pony-music)))
  ([script]
     (let [goto-music
           (flatten
            [0xC3
             (reverse (disect-bytes-2 music-program-start))])]
       (->> script
            (write-RAM
             image-program-start
             (concat
              (display-image-kernel
               image-program-start
               pinkie-pie-mark)
              goto-music))))))
              
(defn-memo activate-media
  ([] (activate-media (write-image-display-program)))
  ([script]
     (->> script
          (transfer-control image-program-start)
          (do-nothing 1800))))

(defn basic-write-move-image! [move target-file]
  (let [actual-keys (set move)
        keys [:a :b :start :select :u :d :l :r]
        button-on-name  ["-size" "5x5" "xc:red"]
        button-off-name ["-size" "5x5" "xc:white"]]
    (apply
     clojure.java.shell/sh
     (flatten 
      ["convert"
       "-size" "64x5" "xc:white"
       (map
        (fn [index]
          (let [position ["-geometry"
                          (str "+" (* index 8)
                               "+" "0") "-composite"]]
            (if (actual-keys (keys index))
              [button-on-name  position]
              [button-off-name position])))
        (range (count keys)))
       (.getCanonicalPath target-file)]))))

(defn basic-composite-frames-command
  [move-image screenshot ram-map target]
  ["convert"
   "-size" "318x276" "xc:white"
   move-image   "-geometry" "+30+225" "-composite"
   screenshot   "-geometry" "+10+10"  "-composite"
   ram-map      "-geometry" "+180+10" "-composite"
   target])

(defn basic-generate-composite-frames!
  [^File rendered-dir]
  (let [final-frames (File. rendered-dir "final")
        _ (.mkdir final-frames)
        move-images (file-names (File. rendered-dir "moves"))
        screenshots (file-names (File. rendered-dir "frames"))
        ram-maps    (file-names (File. rendered-dir "ram-map"))
        targets (map #(.getCanonicalPath
                       (File. final-frames (format "%07d.png" %)))
                     (range (count screenshots)))]
    (dorun
     (map
      (comp
       (partial apply clojure.java.shell/sh)
       flatten
       (partial apply basic-composite-frames-command)
       (fn [a b c d] (println d) [a b c d]))
      move-images screenshots ram-maps targets))))

(def basic-render-dir
  (File. user-home "proj/vba-clojure/render/basic"))

(defn basic-steps [] (first (activate-media)))

(defn basic-step-1! []
  (render-files!
   basic-render-dir
   basic-write-move-image!
   true
   (root) (basic-steps)))

(defn basic-step-2! []
  (basic-generate-composite-frames!
   basic-render-dir))

(defn basic-step-3! [] (final-cut! basic-render-dir))
